/* tslint:disable */
/* eslint-disable */

/* auto-generated by NAPI-RS */

export declare function fromHtml(htmlText: string): string
export declare function fromFile(htmlFilepath: string): string
export declare function fromHtmlToFile(htmlText: string, markdownFilepath: string, overwrites: boolean): void
export declare function fromFileToFile(htmlFilepath: string, markdownFilepath: string, overwrites: boolean): void
